function [ resps, Econt ] = Estep(data, betas, T, noiseparam1, noiseparam2)
% E-STEP: Given the current cluster centroids, find out for every data item
%         from which cluser set it was probably generated

global PRM_MAC;


% mask for 1s in data
mask = (data==1);
clear data

betaL = combineBetas(betas, PRM_MAC.poss_assignments);
clear betas

epsilon = noiseparam1;
r       = noiseparam2;
betaL = epsilon*(1-r) + (1-epsilon)*betaL;

% Avoid numerical problems by division by ("almost") zero
% prepares for log(  betaL):
betaL(  betaL(:)<=eps('single')) =   eps('single');
% prepares for log(1-betaL):
betaL(1-betaL(:)< eps('single')) = 1-eps('single');

% compute risk contributions of 1es
r1 = mask*log(1-betaL') ; % includes sum over features j

% mask for 0s in data
mask = (1-mask);
% compute risk contributions of 0es and add to r1
resps = r1 + mask*log(betaL'); % includes sum over features j


% subtract maximal exponent for numerical stability
max_logresp = max(resps, [], 2);
resps = resps - repmat(max_logresp, [ 1, PRM_MAC.NassSets ]);
% compute unnormalized resps
resps = exp( resps/T );

% normalize
resps = resps .* repmat(1 ./ sum(resps, 2), [1 PRM_MAC.NassSets] );

% continue if the assignments of all objects are not yet sharply peaked at
% one cluster set
if nargout > 1
    maxResp = max(resps, [], 2);
    Econt = (1-min(maxResp) > PRM_MAC.Estep.contThresh);
end


end