function F = F_mix(data, betas, poss_assignments, T, epsilon, r)

%compute proxy centroids for each assignment set
%  ( betaL will always be used together with (1-epsilon) )
betaL = (1-epsilon)*combineBetas(betas, poss_assignments);
  
%mask
m1 = (data==1);
m0 = (1-m1);
clear data; 

vek_x1 =  epsilon*r     - betaL + (1-epsilon) ;
vek_x0 =  epsilon*(1-r) + betaL               ;

%takes care of 0*Inf problem
if any(isnan(vek_x0(:))) || any(isinf(vek_x0(:)))    
    vek_x0( isinf(vek_x0) ) = 1e60;
    vek_x0( isnan(vek_x0) ) = 1e60;
end
if any(isnan(vek_x1(:))) || any(isinf(vek_x1(:)))    
    vek_x1( isinf(vek_x1) ) = 1e60;
    vek_x1( isnan(vek_x1) ) = 1e60;
end

% savety-check for log(0)
vek_x1(vek_x1==0) = 1e-40;
vek_x0(vek_x0==0) = 1e-40;

% includes sum over dimensions 
ciL = exp( ( m1*log(vek_x1') + m0*log(vek_x0') ) /T ); 

%sum over assignment-sets
ciL = sum(  ciL, 2  );

if any(ciL==0) 
    F = Inf;
else
    %sum over objects
    F = -T * sum( log( ciL ), 1 ); 
end

end