function [ therisk, minRiskIndex ] = ...
    Risk(data, betas, noiseparam1, noiseparam2)

% computes neg. log. likelihood of data given model params betas, 
%   noiseparam1, and noiseparam2

global PRM_MAC;


% mask for 1s in data
Ndata = numel(data);
mask = (data==1);
clear data

betaL = combineBetas(betas, PRM_MAC.poss_assignments);
clear betas

% compute the entries of the log's
epsilon = noiseparam1;
r       = noiseparam2;
betaL = epsilon*(1-r) + (1-epsilon)*betaL;


% Avoid numerical problems by division by ("almost") zero
% prepares for log(  betaL):
betaL(  betaL(:)<=eps('single')) =   eps('single');
% prepares for log(1-betaL):
betaL(1-betaL(:)< eps('single')) = 1-eps('single');

%compute risk contributions of 1es
r1 = mask*log(1-betaL') ; % includes sum over features j

%mask for 0s in data
mask = (1-mask);
%compute risk contributions of 0es and add to r1
resps = r1 + mask*log(betaL'); % includes sum over features j

%subtract maximal exponent for numerical stability
[ max_logresp, minRiskIndex ] = max(resps, [], 2);
gammas = resps - repmat(max_logresp, [ 1, PRM_MAC.NassSets ]);
%compute unnormalized resps
gammas = exp( gammas );

%normalize
gammas = gammas .* repmat(1 ./ sum(gammas, 2), [1 PRM_MAC.NassSets] );
therisk = -sum(sum( gammas .* resps )) / Ndata;


end