function betaL = combineBetas(betas, poss_assignments)
% combineBetas(betas, poss_assignments) computes proxy centroids out of the
% centroid matrix 'beta' and the matrix of all possible centroid
% combinations 'poss_assignments'.


Ndim = size(betas, 2);
Nlabelsets = size( poss_assignments, 1);

% construct combined beta for each assignment-set
beta_blow = permute(single(betas),[1 3 2]);
clear betas;
beta_blow = repmat(beta_blow, [1 Nlabelsets 1]);

% ignore betas whose rows do not belong to assignment-set 
poss_assignments_blow = repmat(poss_assignments', [1 1 Ndim]);
beta_blow( poss_assignments_blow ==0 ) = 1;
clear poss_assignments_blow;

% perform product over all betas with betas in same assignmentset
betaL = squeeze( prod(beta_blow,1) );
clear beta_blow;

if Nlabelsets == 1
    % if there is only one label set, the 'squeeze' makes a column vector
    % out of the row vector. This has to be undone in order for the rest to
    % work correctly.
    betaL = betaL';
end

end