function z_ik = convertResult(z_Lk, z_iL)
% z_ik = convertResult(z_Lk, z_iL)
% converts z_iL, a list of assignments between objects i and assignment-
% sets L, and z_Lk, a binary matrix for the mapping of assignment-sets to
% clusters, to direct assignments z_ik between objects and clusters.
%
% input:
%   - z_Lk: a binary matrix, where a 1 at row L and column k indicates the
%       membership of cluster k to assignment-set L. More than one cluster
%       can belong to one assignment-set
%
%   - z_iL: a vector of integers>0, where a number at position i indicates
%       the index L of the assignment-set that object i belongs to
%
% output: 
%   - z_ik: a binary matrix, where a 1 at row i and column k indicates
%       that object i is assigned to cluster k. An object can be assigned
%       to multiple clusters


Nobj = size(z_iL,1);
K = size(z_Lk,2);

% pre-allocate
z_ik = zeros(Nobj, K);
% convert
for i=1:Nobj
    z_ik(i,:) = z_Lk( z_iL(i) ,:);
end

end