function LL = generateAssignmentSets(sourcecount, Degrees)
% Sourcecount: Number of sources (clusters)
% Degrees: Set possible number of clusters a datapoint might belong to 

% catch errors
if max(Degrees)> sourcecount;
    disp(['generateAssignmentSets: You can not take ' ...
        num2str(sourcecount) ' out of ' num2str(Degrees) 'clusters!']);
    LL=[];
elseif (isempty(Degrees)) || (min(Degrees) < 0)  || ...
        (sum(round(Degrees)==Degrees) ~= length(Degrees))          
    disp('generateAssignmentSets: Please take positive integers as degrees');
    LL=[];       
% actual call
else 
    LL = createAssSet(sourcecount, Degrees);
end

end


function LL = createAssSet(Nsrc, Degrees)
LL = [];
NDegrees = length(Degrees);

for deg = 1 : NDegrees
    %recursive call
    LL = [ LL; createAssSetRec(Nsrc, Degrees(deg)) ];
end

end


function LL = createAssSetRec(Nsrc, degree)
% recursive function
if degree > Nsrc || Nsrc == 0; LL = [];
elseif degree == 0; LL = zeros(1, Nsrc);
elseif Nsrc == degree; LL = ones(1, Nsrc);
else
    % there are two choices: use the current source ...
    ml1 = createAssSetRec(Nsrc-1, degree-1);
    % ... or not
    ml2 = createAssSetRec(Nsrc-1, degree);
    % assemble
    mlh1 = size(ml1,1);
    mlh2 = size(ml2,1);
    LL = [ ones(mlh1,1) ml1; zeros(mlh2,1) ml2 ];
end

end