% setMACparams.m
% parameters for the MAC algorithm

% You are discuraged to change anything in this file unless you know
% what you are doing. Please see the file startMAC.m for basic parameters.
%
% Separated in (1) main params and (2) more specialised/sophisticated

% this data-structure will contain all parameters
% it will be visible for the optimizer 'optMAC_DA'
global PRM_MAC

% Parameters for the deterministic annealing scheme
% (1) main params
% Should the noise parameters also be inferred? If the update-flag is set
% to false, the corresponding noise parameter will be fixed at the initial
% value.
PRM_MAC.DetAnn.update_eps = true;
PRM_MAC.DetAnn.update_r = true;

% initialization of noise params
PRM_MAC.DetAnn.start_eps = 0.05;
PRM_MAC.DetAnn.start_r = 0.5;

% -- cooling scheme
% valid choices for Tmax and Tmin and Tfac: 'adapt' or a number. Adapt
% choosing a starting temperature according to Tmax = TmaxFac * l_max,
% where l_max is the largest eigenvalue of the data matrix. This is adapted
% from (Rose, 98).
PRM_MAC.DetAnn.Tmax = 'adapt';
PRM_MAC.DetAnn.TmaxFac = 2.5;
% give the cooling factor Tfac as a number <1 or 'adapt'
PRM_MAC.DetAnn.Tfac = 'adapt';
% factors for the adaptive cooling scheme (only effective if Tfac=true):
PRM_MAC.DetAnn.TfacFast = 0.7; %decrease 4 acceleration
PRM_MAC.DetAnn.TfacSlow = 0.9; %decrease 4 acceleration
% largest number of M-step iterations without changing to the slow cooling
% factor
PRM_MAC.DetAnn.TfacItThresh = 2;

% prevents centroids from becoming pairwise equal
PRM_MAC.DetAnn.avoidBetaCollapse = true;


% -- stopping criteria for DA main-loop
% stop if the largest responsibility for every data item is within
% [ 1, 1-contThresh ]
PRM_MAC.Estep.contThresh = 1e-6;
% stop if T=Tmin is reached
PRM_MAC.DetAnn.Tmin = 'adapt';


% -- output during optimization
PRM_MAC.trace.showT = true; %display temperature
PRM_MAC.trace.DAplotResps = true; %plot responsibilities
PRM_MAC.trace.DAillustrate = true; %plot decomposition in u and z
PRM_MAC.trace.DAillustrateRoundBeta= false; %rounds centroids for the plot

% -- log all steps of the annealing optimization?
PRM_MAC.trace.logData = false;


% (2) more specialised/sophisticated  params
% when to stop M-step repetition?
PRM_MAC.Mstep.changeTol = 0.025;

% how close to 0 should bisection search stop?
PRM_MAC.Mstep.beta.Tol = 0.0001;
PRM_MAC.Mstep.eps.Tol = 0.001;
PRM_MAC.Mstep.r.Tol = 0.001;

% decrease epsilon update tolerance at low T (zoom in)?
PRM_MAC.Mstep.eps.decreaseTol = true; % decrease Tol at low temperature
PRM_MAC.Mstep.eps.decreaseT = 10; % under which T is tol decreased
PRM_MAC.Mstep.eps.decreaseFac = 10; % factor by which tol is decreased
PRM_MAC.Mstep.r.decreaseTol  = true;
PRM_MAC.Mstep.r.decreaseT = 10;
PRM_MAC.Mstep.r.decreaseFac = 10;

% round beta values for estimation of the noise params
PRM_MAC.Mstep.eps.roundBetas = false;
PRM_MAC.Mstep.r.roundBetas = false;

% semi-vectorization granularity for noise updates
% - an integer > 0;
% - controls granularity of  vectorization in noise updates;
% - set higher if you have more MEM, decrease if you get outOfMEM messages
PRM_MAC.Mstep.vectorisationGranul = 5e8;

% Compute the set of possible assignment sets.
% Change this e.g. if you want to allow only particular combination of
% clusters.
if ~exist('Nsrc', 'var') || ~exist('Degrees', 'var')
    error('Please specify both ''Nsrc'' and ''Degrees''.');
end
if useAllcombisFlag
    PRM_MAC.poss_assignments = generateAssignmentSets(Nsrc, Degrees);
else
    PRM_MAC.poss_assignments =...
        generateSparseAssignmentSets(Nsrc, Degrees, multFrac);
end
PRM_MAC.NassSets = size(PRM_MAC.poss_assignments, 1);


