function b = update_mix_beta_newton(data,...
    resps, my_labels, betas, dim, class, epsilon, r)
% Newton update of a single beta parameter
% the update derivations used in subfunction dF_and_d2F_mix_betas are
% provided in the paper

global PRM_MAC;

xdim = data(:, dim);

sumResps_xdim1 = sum(resps(xdim==1,:),1);
sumResps_xdim0 = sum(resps(xdim==0,:),1);

beta_prev = betas(class, dim);


%Newton iterations
b =beta_prev;
converg = 100;
counter = 0;
while converg > PRM_MAC.Mstep.beta.Tol && counter < 10
    [dFdbeta d2Fdbeta2] = dF_and_d2F_mix_betas(sumResps_xdim1, sumResps_xdim0, my_labels, betas, b, dim, class, epsilon, r);
    
    b_old =b ;
    %newton step
    b = min([0.9999999 max( [0.00000001 (b-dFdbeta/d2Fdbeta2)] )]);
    
    % convergence criterion
    converg = abs(b-b_old);
    counter = counter+1;
end

end



function [dF d2F] = dF_and_d2F_mix_betas(sumGammas_xdim1, sumGammas_xdim0, mylabelsets, betas, mybeta,...
    dim, class, epsilon, r)
% dF_mix_betas computes the derivative of the free energy with respect to
% one model parameter 'mybeta' (beta_{pq} in the paper).

% notation conversion (here <=> JMLR-paper)
%    dim   <=> q
%    class <=> p

% pretend that betapq has value mybeta
mbetas = betas(:,dim);
mbetas(class) = mybeta;

%select labelsets containing class p
Nlabelsets = size( mylabelsets, 1);

%%%%%%%% contruct combined beta for each assignment-set
beta_blow = repmat(mbetas, [1 Nlabelsets]);
% ignore betas whose classes do not belong to assignment-set
beta_blow(mylabelsets'==0)  = 1;
% perform product over all betas with classes in same assignment-set
betaL = prod(beta_blow,1);
% contruct combined beta for each assignment-set without contribution of
% current centroid 'p'
% delete row with current centroid 'p'
beta_blow(class,:) = 1;
% perform product over all betas with classes in same assignment-set
betaLp = prod(beta_blow,1);

% for speed; will be used twice in the next two lines:
betaL = (1-epsilon)*betaL;

% perform product of prefactors with betaLP for the cases x=1 and x=0
prefac_x0 = (epsilon*(1-r) + betaL                ); % x=0
prefac_x1 = (epsilon*r     - betaL + (1-epsilon)  ); % x=1

% substitute 0s with very small number to avoid division by 0
prefac_x0( prefac_x0(:) == 0 ) = 1e-20;
prefac_x1( prefac_x1(:) == 0 ) = 1e-20;

prefac_x1 = -betaLp ./ prefac_x1;
prefac_x0 =  betaLp ./ prefac_x0;

dF = (1-epsilon)*(-sumGammas_xdim1*prefac_x1' ...
    -sumGammas_xdim0*prefac_x0' );

d2F = (1-epsilon)*(1-epsilon)*(sumGammas_xdim1*(prefac_x1.*prefac_x1)' ...
    +sumGammas_xdim0*(prefac_x0.*prefac_x0)' );

end




