function r = update_mix_r_bisect(resps, betas, epsilon, data, T)
% bisection search for optimal r param
% would be fater with Newton. might be updated one day...
% the derivative used in the subfunction is provided in the paper

global PRM_MAC

% save memory
betas = single(betas);
if PRM_MAC.Mstep.r.roundBetas
    betas = round(betas);
end

data = single(data);
betaLj = combineBetas(betas, PRM_MAC.poss_assignments);

% how close to zero shall bisection search stop?
tol = PRM_MAC.Mstep.r.Tol;
if PRM_MAC.Mstep.r.decreaseTol && (T<PRM_MAC.Mstep.r.decreaseT)
    tol = tol / PRM_MAC.Mstep.r.decreaseFac;
end

clear betas

%% bisection search
options = optimset('MaxIter', 50, 'TolX', tol);

try
    r = fzero(@(x)dF_mix_r_global(resps, betaLj, x, data, epsilon, ...
        PRM_MAC.Mstep.vectorisationGranul), [eps 1-eps], options);
catch ME
    if strcmp(ME.identifier, 'MATLAB:fzero:ValuesAtEndPtsSameSign' )
        % no sign change in the interval [ 0; 1 ]
        % searching for smallest derivation
        
        %test for lower interval bound
        dF_0 = dF_mix_r_global(resps, betaLj, eps, data, epsilon,...
            PRM_MAC.Mstep.vectorisationGranul);
        if abs(dF_0) < PRM_MAC.Mstep.r.Tol
            r = eps;
        else
            %test for upper interval bound
            dF_1 = dF_mix_r_global(resps, betaLj, 1-eps, data, epsilon,...
                PRM_MAC.Mstep.vectorisationGranul);
            %     if dF_1 == 0
            if abs(dF_1) < PRM_MAC.Mstep.r.Tol
                r = 1-eps;
            else
                % pick value with smallest derivation
                if abs(dF_0) < abs(dF_1)
                    r = eps;
                else
                    r = 1-eps;
                end
            end
        end
        
    else
        % if fzero failed because of another error
        disp('update_mix_r_bisec: fzero failed with error:');
        disp(ME.message);
    end
    
end


end



function dF = dF_mix_r_global(resps, betaLj, r, data, epsilon, vectGran)
% computes the derivation of the free energy w.r.t. epsilon as described in
% Eq.7 of ICML paper

[ Nobj Nfeat ] = size(data);

% prefactors for the cases x=1 and x=0
prefac_x0 = epsilon*(1-r)+ (1-epsilon)*(  betaLj); % x=0
prefac_x1 = epsilon*r    + (1-epsilon)*(1-betaLj); % x=1

% avoid 0
prefac_x0( prefac_x0(:)<=eps ) = 1e-20;
prefac_x1( prefac_x1(:)<=eps ) = 1e-20;

prefac_x0 =  1 ./ prefac_x0; % Eq. 7, h_{Lj0}
prefac_x1 = -1 ./ prefac_x1; % Eq. 7, h_{Lj1}

dF  = zeros(1,Nfeat);
% compute granularity of vectorization
gran = max([2 round( vectGran /Nobj /size(resps,2))]);

restind = rem(Nfeat,gran);
if restind>0
    startInd = Nfeat-restind+1;
    endInd = Nfeat;
    % indicator for ones and zeros:
    mask = repmat( data(:, startInd:endInd), [1 1 2]);  
    mask(:,:,2) = 1-mask(:,:,1);    % indicates the zeros
    
    tmp  = cat(3, resps * prefac_x1(:,startInd:endInd),...
        resps * prefac_x0(:, startInd:endInd));
    dF(startInd:endInd)  = sum( sum(tmp .* mask, 3 ),1 );
end


for chunk = ( 0:floor(Nfeat/gran)-1 )*gran+1
    % perform product with resps
    % this includes summing over assignment-set
    
    startInd = chunk;
    endInd = chunk+gran-1;
    % indicator for ones and zeros:
    mask = repmat( data(:, startInd:endInd), [1 1 2] );  
    mask(:,:,2) = 1-mask(:,:,1);    % indicates the zeros
    
    tmp  = cat(3,resps * prefac_x1(:,startInd:endInd), ...
        resps * prefac_x0(:,startInd:endInd));
    
    % pick the x=1 and x=0 contributions and sum over objects
    dF(startInd:endInd) = sum( sum(tmp .* mask, 3 ),1 );
end

% sum over features
dF = real(sum( dF ));

end


