% function visualize_X2ZYfactorization(X,Z,Y,fignr, iteration, lof_flag)
function visualize_X2ZYfactorization( X, Z, Y, sorting, fignr)

% auxiliary function to visualize a clustering result.
%
% input:
%   - X: the data matrix (N x D)
%   - Z: the assignment matrix (N x K)
%   - Y: the cluster centroids (K x D), Y_{k,d} is the probability for a 1
%           at dimension d of centroid k. The function will behave
%           differently if these values are all binary (not \elem[0,1]).
%   - sorting(optional): indicate whether the matrices should be sorted
%       according to the inferred clustering (if sorting=='sorted') or not
%       (otherwise)
%   - fignr(optional): the number of the figure in which the result is to
%       be displayed
%
% where N = Number of data items
%       D = Number of dimensions
%       K = Number of clusters



% save memory
X = single(X);
Y = single(Y);
Z = single(Z);


% check whether centroids are binary or between 0 and 1
if any( (Y(:) ~= 1) .* (Y(:) ~= 0) )
    
    Xmap = ( Z*round(Y) )>0;
    
    if exist('fignr', 'var')
        figure(fignr);
    else figure;
    end
    colormap('gray');
    subplot(4,5, [ 2 3 7 8 ]); imbin(X);
    title('Input Data X', 'FontSize', 16);
    subplot(4,5, [ 4 5 9 10 ]); imagesc(X-Xmap, [-1 1]);
    title('Input Data - Reconstruction', 'FontSize', 16);
    subplot(4,5, [ 11 16 ]); imbin(Z);
    title('Assignments Zs', 'FontSize', 16);
    subplot(4,5, [ 12 13 ]); imagesc(1-Y, [0 1]);
    title('Centroids p(Us=1)', 'FontSize', 16);
    
    subplot(4,5, [ 14 15 19 20 ]); imbin(Xmap);
    title('Reconstruction Xs', 'FontSize', 16);
    subplot(4,5, [ 1 6 ]); axis off
else
    
    %% sort or not
    if (nargin > 3)
        if strcmp(sorting, 'sorted')
            sort = 1;
        else
            sort = 0;
        end
    else
        sort = 0;
    end
    
    %% sort matrices acc. to cluster assignments
    if (isempty(Z) || isempty(Y))
        disp('visualizer: "empty assignment matrices"');
    else
        k = size(Z,2);
        
        if sort
            %sort according to Z
            Za = sortrows([Z X],-(1:k));
            Xa = Za( :, (k+1):size(Za,2) );
            Za( :, (k+1):size(Za,2)) = [];
            
            clear X Z;
            %sort according 2 Y
            Ya = sortrows( [Y' Xa'], -(1:k) );
            Xa = (Ya(  :, (k+1):size(Ya,2) ))';
            Ya( :, (k+1):size(Ya,2)) = [];
            Ya = Ya';
            clear Y;
        else
            Za = Z; Ya = Y; Xa = X;
            clear X Z Y;
        end        
        
        
        %% visualize
        if exist('fignr', 'var'); figure(fignr); else figure; end
        colormap('gray');
        
        Xs = (Za*Ya)>0;
        
        %close(fignr);
        if exist('fignr', 'var'); figure(fignr); else figure; end
        colormap('gray');
        %set(gcf, 'Position', [ 200 400 1100 700 ] );
        subplot(4,5, [ 2 3 7 8 ]); imbin(Xa);
        title('Input Data X', 'FontSize', 16);
        subplot(4,5, [ 4 5 9 10 ]); imagesc(Xa-Xs, [-1 1]);
        title('Input Data - Reconstruction', 'FontSize', 16);
        subplot(4,5, [ 11 16 ]); imbin(Za);
        title('Assignments Zs', 'FontSize', 16);
        subplot(4,5, [ 12 13 ]); imbin(Ya);
        title('Centroids Us', 'FontSize', 16);
        subplot(4,5, [ 14 15 19 20 ]); imbin(Xs);
        title('Reconstruction Xs', 'FontSize', 16);
        subplot(4,5, [ 1 6 ]); axis off        
    end
end
end